/*
 * JBoss, Home of Professional Open Source
 * Copyright (c) 2010, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.ejb3.security.test.ejbthree1756;

import org.jboss.aspects.common.AOPDeployer;
import org.jboss.ejb3.interceptors.container.BeanContext;
import org.jboss.ejb3.interceptors.direct.DirectContainer;
import org.jboss.security.SecurityAssociation;
import org.jboss.security.client.SecurityClient;
import org.jboss.security.client.SecurityClientFactory;
import org.jboss.security.plugins.JBossAuthenticationManager;
import org.jboss.security.plugins.JBossSecurityContext;
import org.jboss.util.TimedCachePolicy;
import org.jboss.util.naming.NonSerializableFactory;
import org.jnp.server.SingletonNamingServer;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import javax.naming.InitialContext;
import java.security.Principal;

import static org.junit.Assert.assertEquals;

/**
 * @author <a href="cdewolf@redhat.com">Carlo de Wolf</a>
 */
public class RealmMappingUnitTestCase
{
   private static AOPDeployer deployer = new AOPDeployer("jboss-aop.xml");

   @AfterClass
   public static void afterClass()
   {
      deployer.undeploy();
   }

   @BeforeClass
   public static void beforeClass() throws Exception
   {
      System.setProperty("java.security.auth.login.config", RealmMappingUnitTestCase.class.getResource("/jaas.conf").toString());
      SecurityAssociation.setServer();
      
      deployer.deploy();

      new SingletonNamingServer();

      InitialContext ctx = new InitialContext();
      ctx.bind("java:policyRegistration", new SimplePolicyRegistration());

      JBossSecurityContext context = new JBossSecurityContext("TestApp");
      JBossAuthenticationManager authenticationManager = (JBossAuthenticationManager) context.getAuthenticationManager();
      TimedCachePolicy domainCache = new TimedCachePolicy();
      domainCache.create();
      domainCache.start();
      authenticationManager.setCachePolicy(domainCache);
      NonSerializableFactory.rebind(ctx, "java:AuthenticationManager", authenticationManager);
   }

   @Test
   public void testAnonymous() throws Throwable
   {
      DirectContainer<SecuredBean> container = new DirectContainer<SecuredBean>("Test", "SimpleContainer", SecuredBean.class);
      BeanContext<SecuredBean> bean = container.construct();
      Principal callerPrincipal = container.invoke(bean, "getCallerPrincipal");
      assertEquals(CustomPrincipal.class, callerPrincipal.getClass());
      assertEquals("anonymous", callerPrincipal.getName());
   }

   @Test
   public void testTestUser() throws Throwable
   {
      SecurityClient client = SecurityClientFactory.getSecurityClient();
      client.setSimple("testuser", "testpwd");
      client.login();
      try
      {
         DirectContainer<SecuredBean> container = new DirectContainer<SecuredBean>("Test", "SimpleContainer", SecuredBean.class);
         BeanContext<SecuredBean> bean = container.construct();
         Principal callerPrincipal = container.invoke(bean, "getCallerPrincipal");
         assertEquals(CustomPrincipal.class, callerPrincipal.getClass());
         assertEquals("testuser", callerPrincipal.getName());
      }
      finally
      {
         client.logout();
      }
   }
}
