/*
 * JBoss, Home of Professional Open Source
 * Copyright (c) 2010, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.ejb3.security.test.ejbthree1756;

import org.jboss.ejb3.annotation.SecurityDomain;
import org.jboss.ejb3.security.helpers.EJBContextHelper;
import org.jboss.security.*;
import org.jboss.security.audit.AuditManager;
import org.jboss.security.identitytrust.IdentityTrustManager;
import org.jboss.security.javaee.EJBAuthenticationHelper;
import org.jboss.security.javaee.SecurityHelperFactory;
import org.jboss.security.mapping.MappingManager;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.security.auth.Subject;
import java.security.Principal;

/**
 * @author <a href="cdewolf@redhat.com">Carlo de Wolf</a>
 */
public class SecuredBean
{
   public Principal getCallerPrincipal() throws Exception
   {
      SecurityContext prevSC = SecurityContextAssociation.getSecurityContext();
      EJBContextHelper helper = new EJBContextHelper();
      final SecurityContext sc = SecurityContextFactory.createSecurityContext("TestApp");
      if(prevSC != null)
         sc.setSubjectInfo(prevSC.getSubjectInfo());
      sc.setSecurityManagement(new ISecurityManagement()
      {
         public AuthenticationManager getAuthenticationManager(String securityDomain)
         {
            return lookup("java:AuthenticationManager", AuthenticationManager.class);
         }

         public AuthorizationManager getAuthorizationManager(String securityDomain)
         {
            throw new RuntimeException("NYI");
         }

         public MappingManager getMappingManager(String securityDomain)
         {
            throw new RuntimeException("NYI");
         }

         public AuditManager getAuditManager(String securityDomain)
         {
            return null;
         }

         public IdentityTrustManager getIdentityTrustManager(String securityDomain)
         {
            throw new RuntimeException("NYI");
         }
      });
      SecurityContextAssociation.setSecurityContext(sc);
      try
      {
         EJBAuthenticationHelper authenticationHelper = SecurityHelperFactory.getEJBAuthenticationHelper(sc);
         Subject subject = new Subject();
         boolean isValid = authenticationHelper.isValid(subject, "getCallerPrincipal");
         if(!isValid)
         {
            Exception cause = (Exception) sc.getData().get("org.jboss.security.exception");
            throw new RuntimeException("no valid principal", cause);
         }
         authenticationHelper.pushSubjectContext(subject);
         //RealmMapping rm = (RealmMapping) sc.getAuthenticationManager();
         RealmMapping rm = lookup("java:AuthenticationManager", RealmMapping.class);
         SecurityDomain domain = null;
         return helper.getCallerPrincipal(sc, rm, domain);
      }
      finally
      {
         SecurityContextAssociation.clearSecurityContext();
      }
   }

   private static <T> T lookup(String name, Class<T> expectedType)
   {
      try
      {
         return expectedType.cast(new InitialContext().lookup(name));
      }
      catch(NamingException e)
      {
         throw new RuntimeException(e);
      }
   }
}
